from pymock.matcher.matcher import Matcher


class TypeMatcher(Matcher):
    def __init__(self, cls):
        self.__cls = cls

    def __eq__(self, other):
        if isinstance(other, self.__cls):
            return True
        return False

    def __repr__(self):
        return f"<TypeMatcher: {self.__cls}>"




from pymock.action.call_action import CallAction


class RaiseAction(CallAction):
    def __init__(self, exception: Exception):
        self.__exception = exception

    def execute(self):
        raise self.__exception



from typing import Tuple, Optional, Any

from pymock.action.call_action import CallAction
from pymock.action.raise_action import RaiseAction
from pymock.action.return_action import ReturnAction
from pymock.matcher.matcher import Matcher


class CallMock:
    def __init__(self, matchers: Tuple[Any]):
        self.__matchers = matchers
        self.__action: Optional[CallAction] = None

    def returns(self, return_value: Any):
        self.__action = ReturnAction(return_value)

    def raises(self, exception: Exception):
        self.__action = RaiseAction(exception)

    def execute(self):
        return self.__action.execute()

    def has_matchers(self, matchers: Tuple[Matcher]):
        if self.__matchers == matchers:
            return True
        return False



from unittest import TestCase
from unittest.mock import MagicMock

from pymock import PyMock, Is


class TestExamples(TestCase):

    def test_return_object(self):
        class Person:
            pass
        bob = Person()
        mock = PyMock()
        mock.call("bob").returns(bob)
        person_repository = MagicMock()
        person_repository.get_person.side_effect = mock
        self.assertEqual(bob, person_repository.get_person("bob"))

    def test_call_multiple_values(self):
        mock = PyMock()
        mock.call(1, 1).returns(2)
        mock.call(1, 2).returns(3)
        calculator = MagicMock()
        calculator.add.side_effect = mock

        self.assertEqual(2, calculator.add(1, 1))
        self.assertEqual(3, calculator.add(1, 2))

    def test_raise_exception(self):
        class MyException(Exception):
            pass
        mock = PyMock()
        mock.call("invalid").raises(MyException())
        my_object = MagicMock()
        my_object.method.side_effect = mock

        with self.assertRaises(MyException):
            my_object.method("invalid")

    def test_return_magicmock_when_not_matching_any_calls(self):
        mock = PyMock()
        mock.call("won't match this").returns(1)
        my_object = MagicMock()
        my_object.method.side_effect = mock

        self.assertIsInstance(my_object.method("hello"), MagicMock)

    def test_match_instance_type(self):
        class Date:
            pass
        mock = PyMock()
        mock.call(Is.type(Date)).returns(True)
        calendar = MagicMock()
        calendar.has_events.side_effect = mock

        self.assertEqual(True, calendar.has_events(Date()))



from typing import Any

from pymock.action.call_action import CallAction


class ReturnAction(CallAction):
    def __init__(self, return_value: Any):
        self.__return_value: Any = return_value

    def execute(self):
        return self.__return_value



from __future__ import annotations

from typing import List, Any
from unittest.mock import MagicMock

from pymock.matcher.matcher import Matcher
from pymock.call_mock import CallMock


class PyMock:

    def __init__(self):
        self.__call_mocks: List[CallMock] = []

    def call(self, *matchers: Any) -> CallMock:
        entry = CallMock(matchers)
        self.__call_mocks.append(entry)
        return entry

    def __call__(self, *matchers: Matcher, **kwargs):
        for call in self.__call_mocks:
            if call.has_matchers(matchers):
                return call.execute()

        return MagicMock()




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="python-mock",
    version="0.0.2",
    url="https://github.com/diddi-/pymock",
    author="Diddi Oskarsson",
    author_email="diddi@diddi.se",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    include_package_data=True,
    license="MIT",
    install_requires=[
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)
